@@ -47,10 +47,11 @@ namespace
 
 static const map<QString, QString> BUS_NAMES =
 {
-    { "google-drive-scope", "com.canonical.StorageFramework.Provider.ProviderTest" },
-    { "com.canonical.scopes.mcloud_mcloud_mcloud", "com.canonical.StorageFramework.Provider.McloudProvider" },
+    { "storage-provider-test", "com.canonical.StorageFramework.Provider.ProviderTest" },
+    { "storage-provider-mcloud", "com.canonical.StorageFramework.Provider.McloudProvider" },
     { "storage-provider-owncloud", "com.canonical.StorageFramework.Provider.OwnCloud" },
     { "storage-provider-onedrive", "com.canonical.StorageFramework.Provider.OnedriveProvider" },
+    { "storage-provider-gdrive", "com.canonical.StorageFramework.Provider.GdriveProvider" },
 };
 
 }  // namespace
